(***********************************************************************)
(*                                                                     *)
(*              Fast Escape Analysis - Equations solver                *)
(*                                                                     *)
(*                  Bruno Blanchet, ENS Ulm - INRIA Rocquencourt       *)
(*                                                                     *)
(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
(*  Automatique.  Distributed only by permission.                      *)
(*                                                                     *)
(***********************************************************************)

open Ttype

module Nset : Ordset.S with type elt = Ttype.graph_t

module Graph :
sig

type t = graph_t

val incr_mark : unit -> unit
val get_mark : t -> int
val newnode : sons_t -> t option -> t

val get_sons : t -> t array
val get : int -> t -> t
val expand_count : int ref

(* First search : Is letstack possible ? *)
val mark : t -> unit
val marked : t -> int (* The solving function *)
val unmark : t -> unit
(* makes visited <- false *)

val nostack : t (* Special value to say "letstack impossible" *)

(* Second search : Is letstack' possible ? *)
val mark_tail : t -> unit
val tail_marked : t -> int

(* Third search : Where do the parameters escape ? *)
exception EscapeExc
exception ExpandedMet
val to_replace_by_node : t -> t -> unit
val get_node_replacers : t -> Nset.t
val cpt_get_node_replacers : t -> Nset.t
val cpt_get_source_node_replacers : t -> Nset.t

val to_replace_by_type : t -> buildtype_t * newtype_t -> unit
val iter_type_replacers : (buildtype_t * newtype_t -> unit) -> t -> t -> unit
val copy_link : t -> t -> unit
val cpt_copy_link : (buildtype_t * newtype_t -> unit) -> 
  cpt_sons_t -> t -> unit
val cleanup_replace : t -> unit

(* Build the graph *)
val link : t -> t -> unit
(* [link n m] creates an edge from n to m *)
val linksource : t -> t -> unit
(* [linksource n m] creates an edge from each son of n to m *)
val linksons : t -> t -> unit
(* [linksons n m] creates an edge from one node in n's tree to the
   corresponding node in m's tree. Both trees must be the same.
   The only exceptionnal case this function can handle is when one
   tree has no sons whereas the other one still has sons *)
val call : (t * t) list -> (t * t) list -> (t * t) list -> (t * t) list -> unit

(* Copy the graph (useful for inlining) *)
val copy : int -> t -> t
val cleanupallcopies : unit -> unit

end
