(***********************************************************************)
(*                                                                     *)
(*              Fast Escape Analysis - Type declarations               *)
(*                                                                     *)
(*                  Bruno Blanchet, ENS Ulm - INRIA Rocquencourt       *)
(*                                                                     *)
(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
(*  Automatique.  Distributed only by permission.                      *)
(*                                                                     *)
(***********************************************************************)

type t_type_name =
    { d_modname: string; (* To garantee unicity, we use both module name and
			    identifier (with its stamp, each identifier is unique in a file) *)
      d_name: Ident.t }

type dumnodetree_t =
    { mutable d_sons : dumnodetree_t array;
      mutable d_paramson : dumnodetree_t option}

type t_link = 
    NoLink
  | BTlink of buildtype_t
  | NTlink of dumnodetree_t

and buildtype_t =
    { mutable b_desc : buildtype_t_desc;
      mutable b_mark : int;
      mutable n_sons : int;
      mutable b_link : t_link;
      mutable b_link3 : buildtype_t list}

and buildtype_t_desc =
    BTvar
  | BTbadType
  | BTAll
  | BTabstract of t_type_name * t_type_declaration * buildtype_t list
  | BTarrow of int ref * int ref * buildtype_t * buildtype_t
  | BTgroup of t_type_name * (int ref * buildtype_t) list

and t_type_declaration =
    { e_name: t_type_name;
      mutable e_type_params: buildtype_t list;
      mutable e_color: int;
      mutable e_orig_decl: t_orig_decl;
      mutable e_type_kind: buildtype_t}

and t_orig_decl =
    NoDecl
  | OldDecl of Types.type_declaration
  | NewDecl of t_type_declaration

type nodereplacer_t =
    NoRep
  | Escape
  | OneNode of graph_t
  | SeveralNodes of graph_t list

and sons_t =
    Expanded of graph_t array
  | Compact of cpt_sons_t

and cpt_sons_t =
    { mutable cpt_pathEdge : graph_t list;
      mutable cpt_linksource : graph_t Ordset.set;
      mutable cpt_linksons : graph_t Ordset.set;
      mutable cpt_rets : graph_t list;
      mutable cpt_starts : (graph_t * (graph_t * graph_t) list * (graph_t * graph_t) list) list;
      mutable cpt_sons : dumnodetree_t}

and graph_t =
    { num : int;
      temp_mark : int;

      (* First search : Is letstack possible ? *)
      mutable mark : int;
      mutable visited : int;

      (* Second search : Is letstack' possible ? *)
      mutable tail_mark : int;
      mutable tail_visited : int;

      (* Third search : Where do the parameters escape ? *)
      mutable type_replacer : (buildtype_t * newtype_t) list;
      mutable node_replacer : nodereplacer_t;
      mutable replacer_visited : bool;

      (* Edges *)
      mutable pathEdge : graph_t list;
      mutable sourcepathEdge : graph_t list;
      mutable norm : graph_t Ordset.set; (* Normal edges *)
      mutable rets : graph_t list;
      (* For each return, the node returned from *)
      mutable starts : (graph_t * (graph_t * graph_t) list * (graph_t * graph_t) list) list;
      (* For each start, the corresponding call node
         and a list of corresponding return nodes, for each one
             the return node and the exit node
	 and a list of corresponding return nodes which transmit to sons,
	     for each one, the return node and the exit node *)

      (* sons *)
      mutable e_sons : sons_t;
      mutable e_paramson : graph_t option}

and newtype_t = buildtype_t * graph_t

type identinfo_t =
    { mutable type_expr: newtype_t;
      tupled: bool;
      arity: int;
      mutable current_scc: bool;
      mutable current_scc2: bool;
      mutable unknown_tail_call: bool }

type t_env = identinfo_t Ident.tbl

type t_typedecl = t_type_declaration Ident.tbl

type t_hierarchy =
    { env : t_env;
      types : t_typedecl;
      submod : t_submodules;
      submod_types: t_submodules}

and t_hierarchy_mod =
    Module of t_hierarchy
  | Functor of t_hierarchy_mod * t_hierarchy_mod
     (* Pointer to the parameter environment * body
        The parameter environment will be changed when applying the functor
*)

and t_submodules = t_hierarchy_mod Ident.tbl

let emptyhierarchy = { env = Ident.empty;
                       types = Ident.empty;
                       submod = Ident.empty;
                       submod_types = Ident.empty}
